@@ -1,3 +1,33 @@
+/* GDK - The GIMP Drawing Kit
+ * Copyright (C) 1995-1997 Peter Mattis, Spencer Kimball and Josh MacDonald
+ *
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2 of the License, or (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
+ */
+
+/*
+ * Modified by the GTK+ Team and others 1997-2000.  See the AUTHORS
+ * file for a list of people on the GTK+ Team.  See the ChangeLog
+ * files for a list of changes.  These files are distributed with
+ * GTK+ at ftp://ftp.gtk.org/pub/gtk/.
+ */
+
+#if defined(GTK_DISABLE_SINGLE_INCLUDES) && !defined (__GDK_H_INSIDE__) && !defined (GDK_COMPILATION)
+#error "Only <gdk/gdk.h> can be included directly."
+#endif
+
 #ifndef __GDK_DRAWABLE_H__
 #define __GDK_DRAWABLE_H__
 
@@ -84,7 +114,7 @@ struct _GdkDrawableClass
 			  GdkGC	       *gc,
 			  GdkSegment   *segs,
 			  gint		nsegs);
- void (*draw_lines)     (GdkDrawable  *drawable,
+  void (*draw_lines)     (GdkDrawable  *drawable,
 			  GdkGC        *gc,
 			  GdkPoint     *points,
 			  gint          npoints);
@@ -170,10 +200,27 @@ struct _GdkDrawableClass
 
   cairo_surface_t *(*ref_cairo_surface) (GdkDrawable *drawable);
 
+  GdkDrawable *(*get_source_drawable) (GdkDrawable *drawable);
+
+  void         (*set_cairo_clip)      (GdkDrawable *drawable,
+				       cairo_t *cr);
+
+  cairo_surface_t * (*create_cairo_surface) (GdkDrawable *drawable,
+					     int width,
+					     int height);
+
+  void (*draw_drawable_with_src)  (GdkDrawable  *drawable,
+				   GdkGC	       *gc,
+				   GdkDrawable  *src,
+				   gint		xsrc,
+				   gint		ysrc,
+				   gint		xdest,
+				   gint		ydest,
+				   gint		width,
+				   gint		height,
+				   GdkDrawable  *original_src);
+
   /* Padding for future expansion */
-  void         (*_gdk_reserved4)  (void);
-  void         (*_gdk_reserved5)  (void);
-  void         (*_gdk_reserved6)  (void);
   void         (*_gdk_reserved7)  (void);
   void         (*_gdk_reserved9)  (void);
   void         (*_gdk_reserved10) (void);
@@ -182,7 +229,6 @@ struct _GdkDrawableClass
   void         (*_gdk_reserved13) (void);
   void         (*_gdk_reserved14) (void);
   void         (*_gdk_reserved15) (void);
-  void         (*_gdk_reserved16) (void);
 };
 
 struct _GdkTrapezoid
@@ -204,16 +250,19 @@ gpointer        gdk_drawable_get_data     (GdkDrawable    *drawable,
 					   const gchar    *key);
 #endif /* GDK_DISABLE_DEPRECATED */
 
-void            gdk_drawable_get_size     (GdkDrawable	  *drawable,
-					   gint	          *width,
-					   gint  	  *height);
 void	        gdk_drawable_set_colormap (GdkDrawable	  *drawable,
 					   GdkColormap	  *colormap);
 GdkColormap*    gdk_drawable_get_colormap (GdkDrawable	  *drawable);
-GdkVisual*      gdk_drawable_get_visual   (GdkDrawable	  *drawable);
 gint            gdk_drawable_get_depth    (GdkDrawable	  *drawable);
+
+#if !defined (GDK_DISABLE_DEPRECATED)
+void            gdk_drawable_get_size     (GdkDrawable	  *drawable,
+					   gint	          *width,
+					   gint  	  *height);
+GdkVisual*      gdk_drawable_get_visual   (GdkDrawable	  *drawable);
 GdkScreen*	gdk_drawable_get_screen   (GdkDrawable    *drawable);
 GdkDisplay*	gdk_drawable_get_display  (GdkDrawable    *drawable);
+#endif /* GDK_DISABLE_DEPRECATED */
 
 #ifndef GDK_DISABLE_DEPRECATED
 GdkDrawable*    gdk_drawable_ref          (GdkDrawable    *drawable);
@@ -222,6 +271,7 @@ void            gdk_drawable_unref        (GdkDrawable    *drawable);
 
 /* Drawing
  */
+#ifndef GDK_DISABLE_DEPRECATED
 void gdk_draw_point     (GdkDrawable      *drawable,
 			 GdkGC            *gc,
 			 gint              x,
@@ -251,19 +301,14 @@ void gdk_draw_arc       (GdkDrawable      *drawable,
 void gdk_draw_polygon   (GdkDrawable      *drawable,
 			 GdkGC            *gc,
 			 gboolean          filled,
-			 GdkPoint         *points,
-			 gint              npoints);
-#if !defined (GDK_DISABLE_DEPRECATED) || defined (GTK_COMPILATION)
-/* Used by gtk_default_draw_string () */
+			 const GdkPoint   *points,
+			 gint              n_points);
 void gdk_draw_string    (GdkDrawable      *drawable,
 			 GdkFont          *font,
 			 GdkGC            *gc,
 			 gint              x,
 			 gint              y,
 			 const gchar      *string);
-#endif /* !GDK_DISABLE_DEPRECATED || GTK_COMPILATION */
-#if !defined (GDK_DISABLE_DEPRECATED) || defined (GDK_COMPILATION)
-/* Used by gdk_pixmap_draw_text (), gdk_window_draw_text() */
 void gdk_draw_text      (GdkDrawable      *drawable,
 			 GdkFont          *font,
 			 GdkGC            *gc,
@@ -271,7 +316,6 @@ void gdk_draw_text      (GdkDrawable      *drawable,
 			 gint              y,
 			 const gchar      *text,
 			 gint              text_length);
-/* Used by gdk_pixmap_draw_text_wc (), gdk_window_draw_text_wc () */
 void gdk_draw_text_wc   (GdkDrawable      *drawable,
 			 GdkFont          *font,
 			 GdkGC            *gc,
@@ -279,7 +323,6 @@ void gdk_draw_text_wc   (GdkDrawable      *drawable,
 			 gint              y,
 			 const GdkWChar   *text,
 			 gint              text_length);
-#endif /* !GDK_DISABLE_DEPRECATED || GDK_COMPILATION */
 void gdk_draw_drawable  (GdkDrawable      *drawable,
 			 GdkGC            *gc,
 			 GdkDrawable      *src,
@@ -300,19 +343,19 @@ void gdk_draw_image     (GdkDrawable      *drawable,
 			 gint              height);
 void gdk_draw_points    (GdkDrawable      *drawable,
 			 GdkGC            *gc,
-			 GdkPoint         *points,
-			 gint              npoints);
+			 const GdkPoint   *points,
+			 gint              n_points);
 void gdk_draw_segments  (GdkDrawable      *drawable,
 			 GdkGC            *gc,
-			 GdkSegment       *segs,
-			 gint              nsegs);
+			 const GdkSegment *segs,
+			 gint              n_segs);
 void gdk_draw_lines     (GdkDrawable      *drawable,
 			 GdkGC            *gc,
-			 GdkPoint         *points,
-			 gint              npoints);
+			 const GdkPoint   *points,
+			 gint              n_points);
 void gdk_draw_pixbuf    (GdkDrawable      *drawable,
 			 GdkGC            *gc,
-			 GdkPixbuf        *pixbuf,
+			 const GdkPixbuf  *pixbuf,
 			 gint              src_x,
 			 gint              src_y,
 			 gint              dest_x,
@@ -355,22 +398,20 @@ void gdk_draw_layout_with_colors      (GdkDrawable     *drawable,
                                        const GdkColor  *foreground,
                                        const GdkColor  *background);
 
-void gdk_draw_glyphs_transformed (GdkDrawable      *drawable,
-				  GdkGC	           *gc,
-				  PangoMatrix      *matrix,
-				  PangoFont        *font,
-				  gint              x,
-				  gint              y,
-				  PangoGlyphString *glyphs);
-void gdk_draw_trapezoids         (GdkDrawable      *drawable,
-				  GdkGC	           *gc,
-				  GdkTrapezoid     *trapezoids,
-				  gint              n_trapezoids);
+void gdk_draw_glyphs_transformed (GdkDrawable        *drawable,
+				  GdkGC	             *gc,
+				  const PangoMatrix  *matrix,
+				  PangoFont          *font,
+				  gint                x,
+				  gint                y,
+				  PangoGlyphString   *glyphs);
+void gdk_draw_trapezoids         (GdkDrawable        *drawable,
+				  GdkGC	             *gc,
+				  const GdkTrapezoid *trapezoids,
+				  gint                n_trapezoids);
 
-#ifndef GDK_DISABLE_DEPRECATED
 #define gdk_draw_pixmap                gdk_draw_drawable
 #define gdk_draw_bitmap                gdk_draw_drawable
-#endif /* GDK_DISABLE_DEPRECATED */
 
 GdkImage* gdk_drawable_get_image      (GdkDrawable *drawable,
                                        gint         x,
@@ -385,6 +426,7 @@ GdkImage *gdk_drawable_copy_to_image (GdkDrawable  *drawable,
 				      gint          dest_y,
 				      gint          width,
 				      gint          height);
+#endif /* GDK_DISABLE_DEPRECATED */
 
 GdkRegion *gdk_drawable_get_clip_region    (GdkDrawable *drawable);
 GdkRegion *gdk_drawable_get_visible_region (GdkDrawable *drawable);
